unit commu;

interface

uses Forms, Windows, SysUtils;

Const READ_BUFFER = 2048;
      BUFFER_SECONDS = 2;

var PortError: boolean;
  PortErrorMsg: String;

function SerialOpen(com: String; silent: boolean = false): THandle;
function SerialConfigure(hCom: THandle; baud, parity, data, stop: String; silent: boolean = false): Boolean;
function WriteSerial(hCom: THandle; str: String; silent: boolean = false): Boolean;
function ReadSerial(hCom: THandle): AnsiString;

implementation

//==============================================================================

function SerialOpen(com:String; silent:boolean):THandle;
var
  dwError:DWord;
  hCom:THandle;
  CommPortName : array [0..127] of Char;
begin
//StrPCopy(CommPortName,'\\.\'+Copy(fCommPort,1,79))
   //hCom := CreateFile(PChar(com), GENERIC_READ or GENERIC_WRITE, 0, nil, OPEN_EXISTING, 0, 0);
   hCom := CreateFile(StrPCopy(CommPortName,'\\.\'+Copy(com,1,79)), GENERIC_READ or GENERIC_WRITE, 0, nil, OPEN_EXISTING, 0, 0);
   if (hCom = INVALID_HANDLE_VALUE) then
   begin
      dwError:= GetLastError();
      PortErrorMsg := 'Serial open Error ' + com + ' : ' + IntToStr(dwError) + ' (' + SysErrorMessage(dwError) + ')';
      if not silent then
        Application.MessageBox(PChar(PortErrorMsg),PChar(Application.MainForm.Caption), MB_OK);
      PortError:=true;
   end
   else PortError:= false;
   result:= hCom;
end;

//------------------------------------------------------------------------------

function SerialConfigure(hCom: THandle; baud, parity, data, stop: String; silent: boolean): Boolean;
var dwError: DWord;
    dcb, dcbp: TDCB;
    fSuccess: Boolean;
    mult: Integer;
    commtimeouts: TCommtimeouts;
begin
   result:=False;
   fSuccess := GetCommState(hCom, dcb);
   if (not fSuccess) then begin
      dwError := GetLastError();
      PortErrorMsg := 'error on getcommstate: ' + IntToStr(dwError) + ' (' + SysErrorMessage(dwError) + ')';
      if not silent then
        Application.MessageBox(PChar(PortErrorMsg),PChar(Application.MainForm.Caption), MB_OK);
      PortError:= true;
      Exit;
   end
   else PortError:= false;
   
   fSuccess := BuildCommDCB(PChar('baud=' + baud + ' parity=' + parity +
                                  ' data=' + data + ' stop=' + stop), dcbp);
   if (not fSuccess) then begin
      dwError := GetLastError();
      (* Handle the error. *)
      PortErrorMsg := 'error on buildcommdcb: ' +
                                   IntToStr(dwError) + ' (' +
                                   SysErrorMessage(dwError) + ')';
      if not silent then
        Application.MessageBox(PChar(PortErrorMsg),
                                   PChar(Application.MainForm.Caption), MB_OK);
      PortError:= true;
      exit;
   end
   else PortError:= false;

   (* Fill in the DCB: baud, data bits, parity, stop bit. *)
   dcb.BaudRate:= dcbp.BaudRate;
   dcb.ByteSize:= dcbp.ByteSize;
   dcb.Parity:= dcbp.Parity;
   dcb.StopBits:= dcbp.StopBits;
   fSuccess:= SetCommState(hCom, dcb);
   if (not fSuccess) then begin
      dwError := GetLastError();
      (* Handle the error. *)
      PortErrorMsg :='error on setcommstate: ' +
                             IntToStr(dwError) + ' (' +
                             SysErrorMessage(dwError) + ')';
      if not silent then
        Application.MessageBox(PChar(PortErrorMsg),
                             PChar(Application.MainForm.Caption), MB_OK);
      PortError:= true;
      exit;
   end
   else PortError:= false;

   mult:= Trunc(dcbp.BaudRate / 8 * BUFFER_SECONDS / READ_BUFFER);
   fSuccess:= SetupComm(hCom, READ_BUFFER * mult, READ_BUFFER);
   if (not fSuccess) then begin
      dwError := GetLastError();
      (* Handle the error. *)
      PortErrorMsg :='error on setupcomm: ' +
                             IntToStr(dwError) + ' (' +
                             SysErrorMessage(dwError) + ')';
      if not silent then
        Application.MessageBox(PChar(PortErrorMsg),
                             PChar(Application.MainForm.Caption), MB_OK);
      PortError:= true;
      exit;
   end
   else PortError:= false;

   fSuccess:= GetCommTimeouts(hCom, commtimeouts);
   if (not fSuccess) then begin
      dwError := GetLastError();
      PortErrorMsg := 'error on getcommtimeouts: ' +
                             IntToStr(dwError) + ' (' +
                             SysErrorMessage(dwError) + ')';
      (* Handle the error. *)
      if not silent then
        Application.MessageBox(PChar(PortErrorMsg),
                             PChar(Application.MainForm.Caption), MB_OK);
      PortError:= true;
      exit;
   end
   else PortError:= false;

   commtimeouts.ReadIntervalTimeout:= MAXDWORD;
   commtimeouts.ReadTotalTimeoutMultiplier:= 0;
   commtimeouts.ReadTotalTimeoutConstant:= 0;
   fSuccess:= SetCommTimeouts(hCom, commtimeouts);
   if (not fSuccess) then begin
      dwError := GetLastError();
      PortErrorMsg :='error on setcommtimeouts: ' +
                             IntToStr(dwError) + ' (' +
                             SysErrorMessage(dwError) + ')';
      (* Handle the error. *)
      if not silent then
        Application.MessageBox(PChar(PortErrorMsg),
                             PChar(Application.MainForm.Caption), MB_OK);
      PortError:= true;
      exit;
   end
   else PortError:= false;
   //Sleep(1500);
   result:=True;
end;

//------------------------------------------------------------------------------

function WriteSerial(hCom: THandle; str: String; silent: boolean): Boolean;
var tb, bw: DWord;
    Buf: array[1..1024] of AnsiChar;
    res: Boolean;
    dwError: DWord;
    stra: AnsiString;
begin
  stra := AnsiString(str);
   tb:= Length(stra);
   StrPCopy(@Buf[1], stra);
   res:= WriteFile(hCom, Buf, tb, bw, nil);
   if not res then begin
      dwError := GetLastError();
      PortErrorMsg := 'error writing to serial port: ' +
                             IntToStr(dwError) + ' (' +
                             SysErrorMessage(dwError) + ')';
      if not silent then
        Application.MessageBox(PChar(PortErrorMsg),
                             PChar(Application.MainForm.Caption), MB_OK);
     PortError:= true;
     result:=false;
     exit;
   end;
   result:=res;
end;

//------------------------------------------------------------------------------

function ReadSerial(hCom:THandle): AnsiString;
var b: PAnsiChar;
    br: DWord;
begin
   b:= AllocMem(READ_BUFFER + 1);
   br:= 0;
   ReadFile(hCom, b[0], READ_BUFFER, br, nil);
   b[br]:= #0;
   ReadSerial:= StrPas(b);
   FreeMem(b);
end;

//------------------------------------------------------------------------------

end.


