unit main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, OleCtrls, SKYPE4COMLib_TLB, ComCtrls, Menus, ExtCtrls,
  StdCtrls, IniFiles, Registry, jpeg;

const
  CallFailed = [clsFailed, clsMissed, clsRefused, clsBusy, clsCancelled];

type
  // skype incoming messages "queue"
  TSkypeMessageMy = class
  public
    Body: WideString;
    FromHandle: WideString;
  end;

  // record for holding options/config information
  TOptions = record
    // skype stuff
    SkypeOpt: record
      StartSkype: boolean;
      SilentMode: boolean;
      RemoteHandle: string;
    end;
    // gsm stuff
    GsmOpt: record
      // nothing to configure for modem
    end;
    // common stuff .. nothing here also
  end;
  
  TfrmMain = class(TForm)
    sbMain: TStatusBar;
    mnuMain: TMainMenu;
    mnuDivert: TMenuItem;
    mnuSetup: TMenuItem;
    N1: TMenuItem;
    mnuExit: TMenuItem;
    mnuHelp: TMenuItem;
    mnuAbout: TMenuItem;
    TrayIcon: TTrayIcon;
    popTray: TPopupMenu;
    popExit: TMenuItem;
    tmrPomakniFormu: TTimer;
    pnlSkypeLog: TPanel;
    meSkypeLog: TMemo;
    tmrStatusUpdater: TTimer;
    tmrProcessor: TTimer;
    tmrSkypeClientChecker: TTimer;
    tmrGsmChecker: TTimer;
    imgModemGreen: TImage;
    imgModemRed: TImage;
    imgSkypeGreen: TImage;
    imgSkypeRed: TImage;
    imgConnectionOn: TImage;
    imgConnectionOff: TImage;
    pbGsmSignal: TProgressBar;
    lblGsmSignal: TLabel;
    procedure mnuExitClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure TrayIconClick(Sender: TObject);
    procedure popExitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure tmrPomakniFormuTimer(Sender: TObject);
    procedure tmrStatusUpdaterTimer(Sender: TObject);
    procedure SkypeAttachmentStatus(ASender: TObject; Status: TOleEnum);
    procedure SkypeCallStatus(ASender: TObject; const pCall: ICall;
      Status: TOleEnum);
    procedure SkypeMessageStatus(ASender: TObject;
      const pMessage: IChatMessage; Status: TOleEnum);
    procedure FormDestroy(Sender: TObject);
    procedure tmrProcessorTimer(Sender: TObject);
    procedure tmrSkypeClientCheckerTimer(Sender: TObject);
    procedure tmrGsmCheckerTimer(Sender: TObject);
    procedure mnuSetupClick(Sender: TObject);
    procedure mnuAboutClick(Sender: TObject);
  private
    { Private declarations }
    Skype: TSkype;
    gasi_app: boolean;
    gsm_to_skype: integer;
    skype_to_gsm: integer;
    skype_messages: TList;
    hCom: THandle;
    Opcije: TOptions;

    gsm_configured: boolean;
    gsm_timeout_cnt: integer;
    gsm_signal: integer;
    gsm_modem_try: integer;
    gsm_modem_try_idx: integer;

    last_gsm_signal: integer;
    ini: TIniFile;
    appPath: string;

    // skype -> gsm
    function Skype2SMS(msg: TSkypeMessageMy): boolean;
    function Skype2Call(msg: TSkypeMessageMy): boolean;
    function Skype2CUSD(msg: TSkypeMessageMy): boolean;

    // gsm -> skype
    function Call2Skype(tel_nr: string): boolean;
    function SMS2Skype(cmgl: string): boolean;

    // misc functions
    function ReadGSMdata(return_on: string; timeout_50ms: integer): string;
    procedure ConfigureGsmModem;
    procedure LogLine(s: string);
    //procedure Split(const Delimiter: Char; Input: string; const Strings: TStrings);
    procedure StrPreskoci(sta: string; koliko_puta: integer; var u_cemu: string);
    procedure Split2(S: string; Delimiter: string; Strings: TStrings);
    function PosEx(const SubStr, S: string; Offset: Cardinal = 1): Integer;
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

uses commu, setup;

{$R *.dfm}

procedure TfrmMain.mnuExitClick(Sender: TObject);
begin
  if Forms.Application.MessageBox(PChar('Are you sure you want to exit ?'),'Exit',MB_YESNO+MB_ICONQUESTION) = mrNo then exit;
  gasi_app := True;
  Close;
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action:=caFree;
  Skype.Free;
end;

procedure TfrmMain.TrayIconClick(Sender: TObject);
begin
  if frmMain.Visible then
    frmMain.Hide
  else
    frmMain.Show;
end;

procedure TfrmMain.popExitClick(Sender: TObject);
begin
  mnuExitClick(Sender);
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  Skype := TSkype.Create(nil);

  appPath:=ExtractFileDir(ParamStr(0));

  ini:=TIniFile.Create(appPath+'\opcije.ini');
  Opcije.SkypeOpt.RemoteHandle := ini.ReadString('Skype','RemoteHandle','');
  Opcije.SkypeOpt.SilentMode := ini.ReadBool('Skype','SilentMode',false);
  Opcije.SkypeOpt.StartSkype := ini.ReadBool('Skype','StartSkype',false);
  ini.Free; // ini nam ne treba da bude otvoren cijelo vrijeme

  TrayIcon.Hint := frmMain.Caption;
  skype_messages := TList.Create;
  gsm_configured := False;
  gsm_timeout_cnt := 0;
  gsm_modem_try := 0;
  gsm_modem_try_idx := 0;
  hCom:=INVALID_HANDLE_VALUE;
end;

procedure TfrmMain.tmrPomakniFormuTimer(Sender: TObject);
begin
  tmrPomakniFormu.Enabled:=False;

  frmMain.Left := Screen.Width - frmMain.Width - 5;
  frmMain.Top := Screen.Height - frmMain.Height - 35;

  Forms.Application.ProcessMessages;
end;

procedure TfrmMain.tmrStatusUpdaterTimer(Sender: TObject);
begin
  sbMain.Panels[0].Text := 'GSM->Skype: ' + IntToStr(gsm_to_skype);
  sbMain.Panels[1].Text := 'Skype->GSM: ' + IntToStr(skype_to_gsm);

  if (Skype.AttachmentStatus=0) and (Skype.ConnectionStatus=conOnline) and (Opcije.SkypeOpt.RemoteHandle<>'') then
  begin
    sbMain.Panels[2].Text := 'Skype: OK';
    imgSkypeGreen.Visible := true;
    imgSkypeRed.Visible := false;
  end else
  begin
    sbMain.Panels[2].Text := 'Skype: N/A';
    imgSkypeGreen.Visible := false;
    imgSkypeRed.Visible := true;
  end;

  if gsm_configured then
  begin
    sbMain.Panels[3].Text := 'GSM: OK';
    imgModemGreen.Visible := true;
    imgModemRed.Visible := false;
  end else
  begin
    sbMain.Panels[3].Text := 'GSM: N/A';
    imgModemGreen.Visible := false;
    imgModemRed.Visible := true;
  end;

  // treba omogucavati tmrProcessor ako je system ready, ili onemoguciti ako nije
  if (Skype.AttachmentStatus=0) and gsm_configured and (tmrProcessor.Tag=0) then // dok nisam ovaj Tag koristio, zezao me je ovaj timer jer je ukljucivao tmrProcessor i kad nije smio
    tmrProcessor.Enabled:=True
  else
    tmrProcessor.Enabled:=False;

  if tmrProcessor.Enabled then
    sbMain.Panels[4].Text := 'Processor: ON'
  else
  begin
    if tmrProcessor.Tag = 1 then
      sbMain.Panels[4].Text := 'Processor: BUSY'
    else
      sbMain.Panels[4].Text := 'Processor: OFF';
  end;

  Forms.Application.ProcessMessages;
end;

procedure TfrmMain.LogLine(s: string);
begin
  if meSkypeLog.Lines.Count>=255 then meSkypeLog.Clear;
  meSkypeLog.Lines.Add(FormatDateTime('hh:mm:ss',Now)+': '+s);
  Forms.Application.ProcessMessages;
end;

procedure TfrmMain.SkypeAttachmentStatus(ASender: TObject;
  Status: TOleEnum);
begin
  LogLine('&&& Skype App: ' + Skype.Convert.AttachmentStatusToText(Status)); // + ', ' + IntToStr(Status)
end;

procedure TfrmMain.SkypeCallStatus(ASender: TObject; const pCall: ICall;
  Status: TOleEnum);
begin
  // ako nas neko zove, samo mu prekidamo vezu!
  (*
  LogLine('Call Status: ' + Skype.Convert.CallStatusToText(Status) + ', ' + IntToStr(Status));
  LogLine('Call Type: ' + Skype.Convert.CallTypeToText(pCall.type_) + ', ' + IntToStr(pCall.type_));
  *)

  // ako poziv nije odlazni i ako zvonimo
  if (pCall.type_<>3) and (Status=4) then
  begin
    // odbi poziv
    pCall.Finish;
    LogLine('&&& Declining incoming call.');
  end;

  Forms.Application.ProcessMessages;
end;

procedure TfrmMain.SkypeMessageStatus(ASender: TObject;
  const pMessage: IChatMessage; Status: TOleEnum);
var
  msg: TSkypeMessageMy;
begin
  // ako nam dolazi Skype message, onda je treba parsirati i odraditi zeljeni posao
  //LogLine('Message: ' + Skype.Convert.ChatMessageStatusToText(Status) + ', ' + IntToStr(Status));

  // ako je poslana poruka, javi
  if Status=1 then
  begin
    LogLine('&&& Sent Skype message.');
  end;

  // ako je received poruka
  if Status=2 then
  begin
    // ubaci je u niz za parsiranje
    msg := TSkypeMessageMy.Create;
    msg.Body:=pMessage.Body;
    msg.FromHandle:=pMessage.FromHandle;
    skype_messages.Add(msg);
    LogLine('&&& Received Skype command, added to queue.');
  end;

  Forms.Application.ProcessMessages;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  skype_messages.Free;
  CloseHandle(hCom);
end;

procedure TfrmMain.tmrProcessorTimer(Sender: TObject);
var
  msg: TSkypeMessageMy;
  command: string;
  msg_completed: boolean;
  s: string;
  tel_nr: string;
  gsm_signal_: string;
  r: real;
begin
  tmrProcessor.Tag:=1; // ne smije nas niko dirati!
  tmrProcessor.Enabled:=False;

  imgConnectionOff.Visible := false;
  imgConnectionOn.Visible := true;
  Forms.Application.ProcessMessages;

  // 1. Provjeravamo nove Skype poruke
  if skype_messages.Count>0 then
  begin
    msg := TSkypeMessageMy(skype_messages[0]); // uvijek uzimamo najstariju

    LogLine('&&& Parsing Skype message...');

    // ako je dosla od dozvoljenog skype korisnika
    if msg.FromHandle = Opcije.SkypeOpt.RemoteHandle then
    begin
      // parsiranje
      if Length(Trim(msg.Body))>1 then command:=Trim(msg.Body)[1]
      else command:='?';

      // SMS?
      if (command='s') or (command='S') then
      begin
        // salji sms
        msg_completed := Skype2SMS(msg);
      end
      else
      // DIAL?
      if (command='d') or (command='D') then
      begin
        // napravi callback poziv
        msg_completed := Skype2Call(msg);
      end
      else
      // CUSD?
      if (command='c') or (command='C') then
      begin
        // okini CUSD i vrati na skype
        msg_completed := Skype2CUSD(msg);
      end
      else
      // nepoznata komanda?
      begin
        msg_completed:=true; // reci da je odradjena komanda
        s:=    'Commands:' + #13#10;
        s:= s +' Send SMS: S<number> <message>' + #13#10;
        s:= s +' Dial (callback): D<number>' + #13#10;
        s:= s +' Command (CUSD): C<*command#>';
        LogLine('&&& Unknown Skype command, sending back reply...');
        Skype.SendMessage(msg.FromHandle,s);
      end;
    end
    else
    // neki lijevi korisnik nam salje poruku...
    begin
      msg_completed:=true; // da je obrisemo, jer je dosla od unknown korisnika
      LogLine('&&& Unauthorized remote client.');
    end;

    // ako je izvrsna fukcija zavrsila posao...
    if msg_completed then
    begin
      skype_messages.Delete(0); // i uvijek brisemo najstariju koju smo upravo odradili
      LogLine('&&& Parsing done.');
      msg.Free;
    end;
  end;

  Forms.Application.ProcessMessages;

  // 2. Provjeravamo imal novih GSM SMS poruka sa AT+CMGL="ALL"
  repeat // stavljamo "repeat" da bi mogli da uradimo "break" iz ovog komada koda
    //if not gsm_configured then break; // ne treba

    if not WriteSerial(hCom,'AT+CMGL="ALL"' + #13,true) then
    begin
      LogLine('### Error on +CMGL command !');
      break; // ne radi dalje dole, jer nesto nije OK sa modemom
    end;
    s:=ReadGSMdata(#13#10 + 'OK' + #13#10,150);
    if s='' then
    begin
      LogLine('### No response on +CMGL command !');
      break; // ne radi dalje dole, jer nesto nije OK sa modemom
    end
    else if (Pos('+CMGL:',s)>0) then
    // izgleda da ima novih sms poruka, treba ih proslijediti na Skype i obrisati iz mobitela
    begin
      LogLine('$$$ New SMS !');

      // parsiraj poruke i salji na Skype
      SMS2Skype(s);
    end;
  until true; // ne ponavljaj nikad

  Forms.Application.ProcessMessages;

  // 3. Provjeravamo stanje poziva GSM telefona sa AT+CLCC komandom (Ako zvoni: AT+CLCC +CLCC: 1,1,4,0,0,"+38735270735",145)
  repeat // stavljamo "repeat" da bi mogli da uradimo "break" iz ovog komada koda
    //if not gsm_configured then break; // ne treba

    if not WriteSerial(hCom,'AT+CLCC' + #13,true) then
    begin
      LogLine('### Error on +CLCC command !');
      break; // ne radi dalje dole, jer nesto nije OK sa modemom
    end;
    s:=ReadGSMdata('OK' + #13#10,50);
    if s='' then
    begin
      LogLine('### No response on +CLCC command !');
      break; // ne radi dalje dole, jer nesto nije OK sa modemom
    end
    else if (Pos('+CLCC: 1,1,4,',s)>0) then
    // izgleda da telefon ne miruje, haman nas neko zove
    begin
      tel_nr := Copy(s,Pos('"',s)+1,Length(s)); // kopiraj nam sve od prvog "
      Delete(tel_nr,Pos('"',tel_nr),Length(tel_nr)); // obrisi sve poslije " tako da nam ostane samo broj
      tel_nr := Trim(tel_nr);
      LogLine('$$$ RING: ' + tel_nr + ' !');

      // treba uspostaviti Skype poziv prema RemoteClient-u
      Call2Skype(tel_nr);
    end;
  until true; // ne ponavljaj nikad

  // 4. Provjeri jacinu GSM signala...
  repeat // stavljamo "repeat" da bi mogli da uradimo "break" iz ovog komada koda
    //if not gsm_configured then break; // ne treba

    if not WriteSerial(hCom,'AT+CSQ' + #13,true) then
    begin
      LogLine('### Error on +CSQ command !');
      break; // ne radi dalje dole, jer nesto nije OK sa modemom
    end;
    s:=ReadGSMdata(#13#10 + 'OK' + #13#10,50);
    if s='' then
    begin
      LogLine('### No response on +CSQ command !');
      break; // ne radi dalje dole, jer nesto nije OK sa modemom
    end
    else if (Pos('+CSQ: ',s)>0) then
    begin
      gsm_signal_ := Copy(s,Pos(' ',s)+1,Length(s)); // kopiraj nam sve od prvog ' '
      Delete(gsm_signal_,Pos(',',gsm_signal_),Length(gsm_signal_)); // obrisi sve poslije ' ' tako da nam ostane samo broj
      gsm_signal_ := Trim(gsm_signal_);
      gsm_signal := StrToInt(gsm_signal_);
      // javi u log ako je drugaciji od maloprije...
      if last_gsm_signal<>gsm_signal then
      begin
        //LogLine('$$$ Signal level: ' + gsm_signal_ + '.'); // dosadno je ovo kad se ispisuje
      end;
      pbGsmSignal.Position := 99-gsm_signal;
      r := (99-gsm_signal) / 99*100;
      lblGsmSignal.Caption := FloatToStrF(r,ffFixed,5,0) + '%';
      last_gsm_signal := gsm_signal;
    end;
  until true; // ne ponavljaj nikad

  imgConnectionOff.Visible := true;
  imgConnectionOn.Visible := false;
  Forms.Application.ProcessMessages;

  tmrProcessor.Enabled:=True;
  tmrProcessor.Tag:=0; // smiju nas dirat...
end;

procedure TfrmMain.tmrSkypeClientCheckerTimer(Sender: TObject);
begin
  tmrSkypeClientChecker.Enabled := False;

  if not Skype.Client.IsRunning and Opcije.SkypeOpt.StartSkype then
  begin
    try
      // minimized, with splash
      Skype.Client.Start(true,false);
    except
    end;
  end;

  Forms.Application.ProcessMessages;

  if Skype.Client.IsRunning and not (Skype.AttachmentStatus = 0) then
  begin
    try
      // protocol vazda kako treba, no waiting
      Skype.Attach(Skype.Protocol,false);
      if Opcije.SkypeOpt.SilentMode then Skype.SilentMode := True;
    except
    end;
  end;

  Forms.Application.ProcessMessages;

  tmrSkypeClientChecker.Enabled := True;
end;

procedure TfrmMain.tmrGsmCheckerTimer(Sender: TObject);
begin
  tmrGsmChecker.Enabled := False;

  if gsm_timeout_cnt>5 then
  begin
    gsm_configured := False; // "odkonfigurisi" modem ako nije prisutan ili ako se ne javlja nakon jedno 5-10 poziva
    CloseHandle(hCom);
  end;

  if not gsm_configured then
    ConfigureGsmModem;

  tmrGsmChecker.Enabled := True;
end;

procedure TfrmMain.ConfigureGsmModem;
var
  s: string;
  reg: TRegistry;
  sl: TStringList;
  sl2: TStringList;
  i: integer;
  com_port: string;
begin
  // prosurfaj kroz sve kineske GPRS modeme iz registrija i probaj jedan po jedan
  // 5 puta probamo svaki koji nadjemo iz registrija, i idemo na naredni
  Inc(gsm_modem_try); // try counter ++
  if gsm_modem_try>5 then
  begin
    Inc(gsm_modem_try_idx); // naredni modem iz registrija
  end;

  // next portion of code is trying to figure out COM ports of all installed "dealextreme" modems...
  // pronadji modeme iz Registrija i popuni u listu
  reg := TRegistry.Create();
  reg.RootKey := HKEY_LOCAL_MACHINE;
  reg.OpenKeyReadOnly('SYSTEM\ControlSet001\Control\Class\{4D36E96D-E325-11CE-BFC1-08002BE10318}\');
  sl := TStringList.Create();
  sl2 := TStringList.Create();
  reg.GetKeyNames(sl);
  reg.CloseKey;
  // za sve modeme koje smo nasli
  for i:=0 to sl.Count-1 do
  begin
    reg.OpenKeyReadOnly('SYSTEM\ControlSet001\Control\Class\{4D36E96D-E325-11CE-BFC1-08002BE10318}\'+sl[i]+'\');

    // ako je Huawei E1150
    if Pos('usb\vid_12d1&pid_1001', reg.ReadString('MatchingDeviceId')) = 1  then
    begin
      sl2.Add(reg.ReadString('AttachedTo'));
    end;

    // ako je nas kineski
    if reg.ReadString('MatchingDeviceId') = 'usb\vid_067b&pid_0609' then
    begin
      sl2.Add(reg.ReadString('AttachedTo'));
    end;
    reg.CloseKey;
  end;
  sl.Free;

  // nema vise modema? iz pocetka...
  if (gsm_modem_try_idx+1) > sl2.Count then
    gsm_modem_try_idx := 0; // prvi opet

  if sl2.Count = 0 then
    Exit;


  // probaj kineski modem koji je trenutno dosao na red
  com_port := sl2[gsm_modem_try_idx];
  LogLine('### Trying modem at ' + com_port);
  sl2.Free;
  reg.Free;
  //-- we should have figured out COM port by now...

  // otvori COM port koji smo skontali iz registryja
  hCom:=SerialOpen(com_port,true);
  if not PortError then
  begin
    if not SerialConfigure(hCom, '115200', 'N', '8', '1', true) then
    begin
      LogLine('### COM port '+com_port+' init error !');
      CloseHandle(hCom);
      Exit;
    end;
  end
  else
  begin
    LogLine('### COM port '+com_port+' open error !');
    Exit;
  end;

  LogLine('$$$ Initializing GSM...');

  // inicijalizuj GSM modem sa AT komandama...
  // vidi jel ziv modem - 2 puta ga prozovi...
  if not WriteSerial(hCom,'AT' + #13,true) then
  begin
    LogLine('### Error on AT command !');
    CloseHandle(hCom);
    Exit;
  end;
  if not WriteSerial(hCom,'AT' + #13,true) then
  begin
    LogLine('### Error on AT command !');
    CloseHandle(hCom);
    Exit;
  end;
  s:=ReadGSMdata('OK',50);
  if s='' then
  begin
    LogLine('### No response on AT command !');
    CloseHandle(hCom);
    Exit;
  end;

  // trebal pin?
  if not WriteSerial(hCom,'AT+CPIN?' + #13,true) then
  begin
    LogLine('### Error on +CPIN command !');
    CloseHandle(hCom);
    Exit;
  end;
  s:=ReadGSMdata('+CPIN',150);
  if s='' then
  begin
    LogLine('### No response on +CPIN command !');
    CloseHandle(hCom);
    Exit;
  end
  else if not (Pos('READY',s)>0) then
  begin
    LogLine('### PIN not disabled on SIM card !');
    CloseHandle(hCom);
    Exit;
  end;

  // sms text-mode
  if not WriteSerial(hCom,'AT+CMGF=1' + #13,true) then
  begin
    LogLine('### Error on +CMGF command !');
    CloseHandle(hCom);
    Exit;
  end;
  s:=ReadGSMdata('OK',50);
  if s='' then
  begin
    LogLine('### No response on +CMGF command !');
    CloseHandle(hCom);
    Exit;
  end
  else if not (Pos('OK',s)>0) then
  begin
    LogLine('### Error while setting SMS to text mode !');
    CloseHandle(hCom);
    Exit;
  end;

  LogLine('$$$ GSM initialized.');

  gsm_configured := True; // na kraju ako nije bilo gresaka...
end;

function TfrmMain.ReadGSMdata(return_on: string;
  timeout_50ms: integer): string;
var
  s: string;
  i: integer;
begin
  s:='';
  i:=0;
  repeat
    Sleep(50);
    Forms.Application.ProcessMessages;
    s:=s+String(ReadSerial(hCom));
    inc(i);
  until( (Pos(return_on,s)>0) or (i>timeout_50ms) );
  if i>timeout_50ms then
  begin
    s:=''; // naznaci da je bio timeout
    Inc(gsm_timeout_cnt);
  end
  else
  begin
    gsm_timeout_cnt := 0;
  end;
  Result := s;
end;

function TfrmMain.Skype2Call(msg: TSkypeMessageMy): boolean;
var
  tel_nr: string;
  s: string;
  gsm_alerting: boolean;
  i: integer;
  call: ICall;
begin
  tel_nr := Trim(msg.Body);
  Delete(tel_nr,1,1);
  tel_nr := Trim(tel_nr);

  Skype.SendMessage(msg.FromHandle,'Dialing: ' + tel_nr + ' please wait...');

  if Length(tel_nr)<=0 then
  begin
    Skype.SendMessage(msg.FromHandle,'No number to dial.');
    LogLine('$$$ No number to dial.');

    Result := true;
    Exit;
  end;

  Inc(skype_to_gsm);

  gsm_alerting := false;

  // 1. napravi AT poziv na mobilni... i provjeravamo stanje poziva GSM telefona sa AT+CLCC komandom
  repeat // stavljamo "repeat" da bi mogli da uradimo "break" iz ovog komada koda
    if not WriteSerial(hCom,'ATD'+tel_nr+';' + #13,true) then
    begin
      LogLine('### Error on ATD command !');
      break; // ne radi dalje dole, jer nesto nije OK sa modemom
    end;
    s:=ReadGSMdata(#13#10 + 'OK' + #13#10,50);
    if s='' then
    begin
      LogLine('### No response on ATD command !');
      break; // ne radi dalje dole, jer nesto nije OK sa modemom
    end;
    LogLine('$$$ GSM call initiated...');
  until true; // ne ponavljaj nikad

  // 2. Provjeravamo stanje sa AT+CLCC
  repeat
    // prvi moramo proci kroz DIALING: +CLCC: 1,0,2, proceduru
    if not WriteSerial(hCom,'AT+CLCC' + #13,true) then
    begin
      LogLine('### Error on +CLCC command !');
      break; // ne radi dalje dole, jer nesto nije OK sa modemom
    end;
    s:=ReadGSMdata(#13#10 + 'OK' + #13#10,50);
    if s='' then
    begin
      LogLine('### No response on +CLCC command !');
      break; // ne radi dalje dole, jer nesto nije OK sa modemom
    end
    // dobro je, Dialingujemo
    else if (Pos('+CLCC: 1,0,2,',s)>0) then
    begin
    end
    // dobro je, Alertiramo: bas zovemo nekog
    else if (Pos('+CLCC: 1,0,3,',s)>0) then
    begin
      LogLine('$$$ GSM call alerting...');
      gsm_alerting := true;
      break; // iskoci, da mozes kreirati SKYPE poziv sad...
    end
    // nema nikakvog rezultata, znaci da poziv nije uspostavljen odnosno nije ni zazvonio tamo telefon onom koga zovemo
    else
    begin
      LogLine('$$$ Number busy or connection error !');
      gsm_alerting := false;
      Skype.SendMessage(msg.FromHandle,'Number busy or connection error !');
      break; // iskoci jer je haman poziv oboren prije nego sto je uspostavljena veza (BUSY ili neki ERROR)
    end;
  until false; // ponavljaj uvijek! iskocicemo sa break-om gore

  // ako je gsm uspio kreirati poziv, mozemo i mi preko Skypeta
  if gsm_alerting then
  begin
    // ako SLUCAJNO ima aktivnog poziva u toku, prekidaj sve
    for i:=1 to Skype.ActiveCalls.Count do
    begin
      Skype.SendMessage(msg.FromHandle,'Terminating Skype call '+IntToStr(i)+'...');
      LogLine('&&& Terminating Skype call '+IntToStr(i)+'...');
      Skype.ActiveCalls[i].Finish;
    end;

    // 3. Kreiramo Skype poziv...
    LogLine('&&& Placing Skype callback call...');
    call := Skype.PlaceCall(msg.FromHandle,'','','');

    // 4. Moramo ovdje sacekati da se uspostavi Skype poziv, odnosno da ona strana prihvati poziv
    repeat
      Forms.Application.ProcessMessages;
    until (call.Status = clsInProgress) or (call.Status in CallFailed);

    // 5. Vrtimo se i cekamo da bilo ko prekine vezu, pa da mi prekinemo preostalu
    // dok god imamo aktivan Skype poziv...
    while (Skype.ActiveCalls.Count>0) and (call.Status = clsInProgress) do
    begin
      Forms.Application.ProcessMessages;

      // prozivaj mobilni i gledaj jel i na njemu aktivan poziv :)
      // iskacemo kad se desi da nema u rezultatu "+CLCC: " - to znaci da nema nijednog aktivnog poziva
      if not WriteSerial(hCom,'AT+CLCC' + #13,true) then
      begin
        LogLine('### Error on +CLCC command !');
        break; // ne radi dalje dole, jer nesto nije OK sa modemom
      end;
      s:=ReadGSMdata(#13#10 + 'OK' + #13#10,50);
      if s='' then
      begin
        LogLine('### No response on +CLCC command !');
        break; // ne radi dalje dole, jer nesto nije OK sa modemom
      end
      // nema "+CLCC: " stringa?
      else if not (Pos('+CLCC: ',s)>0) then
      begin
        Skype.SendMessage(msg.FromHandle,'GSM detected call end.');
        LogLine('$$$ GSM detected call end.');
        break; // gotovo je, GSM je prekinuo vezu, moramo sve prekinuti...
      end;
    end;
  end;

  // prekidaj Skype konekciju - koliko god ih ima
  for i:=1 to Skype.ActiveCalls.Count do
  begin
    LogLine('&&& Finishing Skype call...');
    Skype.ActiveCalls[i].Finish;
  end;

  LogLine('$$$ GSM hanging up...');
  // spustaj vezu na mobitelu
  if not WriteSerial(hCom,'ATH' + #13,true) then
  begin
    LogLine('### Error on ATH command !');
  end;
  s:=ReadGSMdata(#13#10 + 'OK' + #13#10,50);
  // spustaj vezu na mobitelu jos jednom!
  if not WriteSerial(hCom,'ATH' + #13,true) then
  begin
    LogLine('### Error on ATH command !');
  end;
  s:=ReadGSMdata(#13#10 + 'OK' + #13#10,50);
  Skype.SendMessage(msg.FromHandle,'GSM ended call.');

  Result := True; //?
end;

function TfrmMain.Skype2SMS(msg: TSkypeMessageMy): boolean;
var
  to_nr: string;
  sms: string;
  s: string;
  msg_body: string;
begin
  msg_body:=Trim(msg.Body);
  Delete(msg_body,1,1); // obrisi parametar S
  to_nr := Copy(msg_body,1,Pos(' ' ,msg_body)); // uzmi broj primaoca
  Delete(msg_body,1,Pos(' ',msg_body)); // obrisi broj primaoca tako da nam ostane samo poruka
  sms := msg_body;

  LogLine('$$$ GSM sending SMS...');
  Skype.SendMessage(Opcije.SkypeOpt.RemoteHandle,'GSM sending SMS...');

  if (Length(to_nr)<=0) or (Length(sms)<=0) then
  begin
    LogLine('$$$ SMS sending aborted, no recipient or SMS text.');
    Skype.SendMessage(Opcije.SkypeOpt.RemoteHandle,'SMS sending aborted, no recipient or SMS text.');
    Result := True;
    Exit;
  end;

  if not WriteSerial(hCom,'AT+CMGS="'+to_nr+'"' + #13,true) then
  begin
    LogLine('### Error on +CMGS command !');
    Skype.SendMessage(Opcije.SkypeOpt.RemoteHandle,'GSM not able to send SMS message.');
  end;
  s:=ReadGSMdata('>',60); // 3 sec
  if s='' then
  begin
    LogLine('### No response on +CMGS command !');
    Skype.SendMessage(Opcije.SkypeOpt.RemoteHandle,'GSM not able to send SMS message.');
  end
  // nema ">" stringa?
  else if not (Pos('>',s)>0) then
  begin
    Skype.SendMessage(Opcije.SkypeOpt.RemoteHandle,'GSM not able to send SMS message.');
    LogLine('$$$ GSM not able to send SMS message.');
  end
  else
  // sve ok, sacemo poslat poruku
  begin
    // posalji mu sms text
    if not WriteSerial(hCom,msg_body + #26,true) then
    begin
      LogLine('### Error while inputing SMS text !');
    end;
    s:=ReadGSMdata(#13#10,200); // 10 sekundi ima da posalje poruku
    if s='' then
    begin
      LogLine('### No response after inputing SMS text !');
      Skype.SendMessage(Opcije.SkypeOpt.RemoteHandle,'GSM not able to send SMS message.');
    end
    // nema "+CMGS:" stringa? (nije otisla poruka ok?)
    else if not (Pos('+CMGS:',s)>0) then
    begin
      Skype.SendMessage(Opcije.SkypeOpt.RemoteHandle,'GSM not able to send SMS message.');
      LogLine('$$$ GSM not able to send SMS message.');
    end
    // ima +CMGS stringa? (otisla poruka OK?)
    else if Pos('+CMGS:',s)>0 then
    begin
      Skype.SendMessage(Opcije.SkypeOpt.RemoteHandle,'GSM successfully sent SMS message.');
      LogLine('$$$ GSM successfully sent SMS message.');
      Inc(skype_to_gsm);
    end;
  end;

  Result:=True; //?
end;

function TfrmMain.Call2Skype(tel_nr: string): boolean;
var
  i: integer;
  s: string;
  call: ICall;
  gsm_open: boolean;
begin
  if Skype.ConnectionStatus<>conOnline then
  begin
    Result := False; // error
    Exit; // vrati se jer Skype nije konektovan!
  end;

  Inc(gsm_to_skype);

  LogLine('&&& Diverting call...');
  Skype.SendMessage(Opcije.SkypeOpt.RemoteHandle,'Incoming call from: ' + tel_nr);

  gsm_open := false; // reci da nije javljen gsm

  // ako SLUCAJNO ima aktivnog poziva u toku, prekidaj sve
  for i:=1 to Skype.ActiveCalls.Count do
  begin
    LogLine('&&& Terminating Skype call '+IntToStr(i)+'...');
    Skype.ActiveCalls[i].Finish;
  end;

  // pokreni skype poziv...
  call := Skype.PlaceCall(Opcije.SkypeOpt.RemoteHandle,'','','');

  // cekaj dok se ne uspostavi poziv ... ili odbije
  repeat
    // todo: ovdje staviti neki timer, da se odradi "ATA" na telefonu ako dugo zvonimo, da produzimo zvonjenje? mozda da nece trebati uopste...

    // prozivaj mobilni i gledaj jel jos uvijek zvoni...
    // iskacemo kad se desi da nema u rezultatu "+CLCC: " - to znaci da nema nijednog aktivnog poziva odnosno onaj ko je zvao je odustao ili je predugo zvonio
    ReadSerial(hCom); // dummy read - da ocistimo buffer
    if not WriteSerial(hCom,'AT+CLCC' + #13,true) then
    begin
      LogLine('### Error on +CLCC command !');
      break; // ne radi dalje dole, jer nesto nije OK sa modemom
    end;
    s:=ReadGSMdata(#13#10 + 'OK' + #13#10,50);
    if s='' then
    begin
      LogLine('### No response on +CLCC command !');
      break; // ne radi dalje dole, jer nesto nije OK sa modemom
    end
    // nema "+CLCC: " stringa?
    else if not (Pos('+CLCC: ',s)>0) then
    begin
      Skype.SendMessage(Opcije.SkypeOpt.RemoteHandle,'GSM detected call end.');
      LogLine('$$$ GSM detected call end.');
      break; // gotovo je, GSM je prekinuo vezu, moramo sve prekinuti...
    end;

    Forms.Application.ProcessMessages;
  until (call.Status = clsInProgress) or (call.Status in CallFailed);

  // ako je uspio poziv
  if call.Status = clsInProgress then
  begin
    repeat
      // digni slusalicu...
      if not WriteSerial(hCom,'ATA' + #13,true) then
      begin
        LogLine('### Error on ATA command !');
        break; // ne radi dalje dole, jer nesto nije OK sa modemom
      end;
      s:=ReadGSMdata(#13#10 + 'OK' + #13#10,50);
      if s='' then
      begin
        LogLine('### No response on ATA command !');
        break; // ne radi dalje dole, jer nesto nije OK sa modemom
      end
      // doslo OK ?
      else if (Pos(#13#10 + 'OK' + #13#10,s)>0) then
      begin
        gsm_open := true; // javio se GSM uspjesno
        break;
      end
      else
      begin
        // telefon nije rekao OK, znaci nesto nije ok
        break;
      end;
    until true; // ne ponavljaj sine...
    
    // Vrtimo se i cekamo da bilo ko prekine vezu, pa da mi prekinemo preostalu
    // dok god imamo aktivan Skype poziv...i ako se telefon gore javio uspjesno
    while (Skype.ActiveCalls.Count>0) and (call.Status = clsInProgress) and gsm_open do
    begin
      Forms.Application.ProcessMessages;

      // prozivaj mobilni i gledaj jel i na njemu aktivan poziv :)
      // iskacemo kad se desi da nema u rezultatu "+CLCC: " - to znaci da nema nijednog aktivnog poziva
      if not WriteSerial(hCom,'AT+CLCC' + #13,true) then
      begin
        LogLine('### Error on +CLCC command !');
        break; // ne radi dalje dole, jer nesto nije OK sa modemom
      end;
      s:=ReadGSMdata(#13#10 + 'OK' + #13#10,50);
      if s='' then
      begin
        LogLine('### No response on +CLCC command !');
        break; // ne radi dalje dole, jer nesto nije OK sa modemom
      end
      // nema "+CLCC: " stringa?
      else if not (Pos('+CLCC: ',s)>0) then
      begin
        Skype.SendMessage(Opcije.SkypeOpt.RemoteHandle,'GSM detected call end.');
        LogLine('$$$ GSM detected call end.');
        break; // gotovo je, GSM je prekinuo vezu, moramo sve prekinuti...
      end;
    end;
  end;

  // prekidaj Skype konekciju - koliko god ih ima
  for i:=1 to Skype.ActiveCalls.Count do
  begin
    LogLine('&&& Finishing Skype call...');
    Skype.ActiveCalls[i].Finish;
  end;

  LogLine('$$$ GSM hanging up...');
  // spustaj vezu na mobitelu
  if not WriteSerial(hCom,'ATH' + #13,true) then
  begin
    LogLine('### Error on ATH command !');
  end;
  s:=ReadGSMdata(#13#10 + 'OK' + #13#10,50);
  // spustaj vezu na mobitelu jos jednom!
  if not WriteSerial(hCom,'ATH' + #13,true) then
  begin
    LogLine('### Error on ATH command !');
  end;
  s:=ReadGSMdata(#13#10 + 'OK' + #13#10,50);
  Skype.SendMessage(Opcije.SkypeOpt.RemoteHandle,'GSM ended call.');
 
  Result:=True; // ne tretiram?
end;

function TfrmMain.SMS2Skype(cmgl: string): boolean;
var
  sms: TStringList;
  i: integer;
  s: string;
  m: IChatMessage;

  id: string;
  Timestamp: string;
  Body: string;
  FromNr: string;
begin
  if Skype.ConnectionStatus<>conOnline then
  begin
    Result := False; // error
    Exit; // vrati se jer Skype nije konektovan!
  end;

  Inc(gsm_to_skype);

  LogLine('&&& Parsing SMS...');

  sms := TStringList.Create;
  sms.Clear;
  try
    Split2(cmgl, '+CMGL: ', sms) ;
  finally
  end;

  (*
  AT+CMGL="ALL"
  +CMGL: 1,"REC UNREAD","38761xxxxx9",,"11/05/02,13:26:23+08",145,7
  Test...
  OK
  *)

  // za svaku poruku koja je dosla, salji korisniku na skype i treba je obrisati sa mobitela
  // idemo od 1 - NE ZNAM ZASTO, nemam kad debugirat
  for i:=1 to sms.Count-1 do
  begin
    s := sms[i]; // jedan CMGL

    Delete(s,1,Pos(' ',s)); // brisi sve do ID-a
    id := Copy(s,1,Pos(',',s)-1); // kopiraj sve do zareza
    StrPreskoci('"',3,s);
    FromNr := Copy(s,1,Pos('"',s)-1); // kopiraj broj do kraja
    StrPreskoci(',',2,s);
    Timestamp := Copy(s,2,Pos('",',s)-2); // kopiraj timestamp do kraja
    StrPreskoci(#10,1,s); // brisemo sad sve do CR LF da uzmemo poruku
    if Pos(#13#10 + 'OK' + #13#10,s)>0 then
      Body := Copy(s,1,Pos(#13#10 + 'OK' + #13#10,s)) // uzimamo sve do onog zadnjeg klinca OK.. posto je ovo zadnja poruka u nizu
    else
      Body := s;

    // proslijedi je na skype korisniku
    s:=  'New SMS'+#13#10;
    s:=s+' From: ' + FromNr + #13#10;
    s:=s+' Timestamp: ' + Timestamp + #13#10;
    s:=s+' Message: ' + Body;
    LogLine(s);
    m := Skype.SendMessage(Opcije.SkypeOpt.RemoteHandle,s + #13#10);
    //if m.Status .. blabla - ne provjeravam ovo

    // obrisi je sad sa mobitela
    if not WriteSerial(hCom,'AT+CMGD='+id + #13,true) then
    begin
      LogLine('### Error on +CMGD command !');
    end;
    s:=ReadGSMdata('OK',100); // 5 sekundi
    if s='' then
    begin
      LogLine('### No response on +CMGD command !');
    end
    // nema "OK" rezultata?
    else if not (Pos('OK',s)>0) then
    begin
      Skype.SendMessage(Opcije.SkypeOpt.RemoteHandle,'SMS deletion failed in the phone.');
      LogLine('$$$ SMS deletion failed in the phone.');
    end;
  end;

  sms.Free;

  Result:=True; // ne tretiram?
end;

function TfrmMain.PosEx(const SubStr, S: string;
  Offset: Cardinal): Integer;
var
  Tmp: PChar;
begin
  Result := 0;
  if (Offset > Cardinal(Length(S))) then
    exit;
  if Offset = 1 then
    Result := Pos(SubStr, S)
  else
  begin
    Tmp := StrPos(PChar(@S[Offset]), PChar(SubStr));
    if Tmp = nil then
      exit;
    Result := Cardinal(Tmp - PChar(@S[Offset])) + Offset;
  end;
end;

// splita po stringu
procedure TfrmMain.Split2(S, Delimiter: string; Strings: TStrings);
var
  P, OldP: integer;
  Token: string;
begin
  // Prevent any errors due to bogus parameters
  if (Strings = nil) or (Length(S) = 0) or (Length(Delimiter) = 0) then
    exit;
  P := Pos(Delimiter, S);
  OldP := 1;
  Strings.Clear;
  while P > 0 do
  begin
    Token := Copy(S, OldP, P-OldP);
    //if Token<>'' then
      Strings.Add(Token);
    // Don't call delete, instead save off P and search from
    // P + 1 to the end of S
    OldP := P + 1;
    P := PosEx(Delimiter, S, OldP);
  end;
  if P = 0 then
    Strings.Add(Copy(S, OldP, Length(S)));
end;

// splita po charu
(*
procedure TfrmMain.Split(const Delimiter: Char; Input: string; const Strings: TStrings);
begin
  Assert(Assigned(Strings)) ;
  Strings.Clear;
  Strings.Delimiter := Delimiter;
  Strings.DelimitedText := Input;
end;
*)

procedure TfrmMain.StrPreskoci(sta: string; koliko_puta: integer;
  var u_cemu: string);
var
  i: integer;
begin
  for i:=1 to koliko_puta do
  begin
    Delete(u_cemu,1,Pos(sta,u_cemu));
  end;
end;

function TfrmMain.Skype2CUSD(msg: TSkypeMessageMy): boolean;
var
  command: string;
  s: string;
begin
  //primjer: AT+CUSD=1,"*101#",15

  Inc(skype_to_gsm);

  command := Trim(msg.Body);
  Delete(command,1,1);
  command := Trim(command);

  Skype.SendMessage(msg.FromHandle,'$$$ Issuing CUSD command: ' + command + '.');

  if Length(command)<=0 then
  begin
    Skype.SendMessage(msg.FromHandle,'No CUSD command to issue.');
    LogLine('### No CUSD command to issue.');
    Result := true;
    Exit;
  end;

  // 1. okini komandu i sacekaj da ti je mobitel vrati ECHO-om
  repeat // stavljamo "repeat" da bi mogli da uradimo "break" iz ovog komada koda
    if not WriteSerial(hCom,'AT+CUSD=1,"' + command + '",15' + #13,true) then
    begin
      LogLine('### Error on +CUSD command !');
      break; // ne radi dalje dole, jer nesto nije OK sa modemom
    end;
    s:=ReadGSMdata(',15'+#13,50); // cekaj da on vrati ECHO-om nazad ono sto si mu poslao...
    if s='' then
    begin
      LogLine('### No response on +CUSD command !');
      break; // ne radi dalje dole, jer nesto nije OK sa modemom
    end;

    LogLine('$$$ Waiting for CUSD response...');
    Skype.SendMessage(msg.FromHandle,'$$$ CUSD command initiated, waiting for response 8 sec max...');

    s:=ReadGSMdata(#13#10 + 'OK' + #13#10,160); // cekaj rezultat jedno 8 sekundi
    if s='' then
    begin
      LogLine('### No network response on +CUSD command !');
      break; // ne radi dalje dole, jer nesto nije OK sa modemom
    end
    else
    begin
      StrPreskoci('"',1,s);
      s := Copy(s,1,Pos('",',s)-1);
      // dumpaj je korisniku na Skype
      LogLine('$$$ GSM CUSD response: ' + s);
      Skype.SendMessage(msg.FromHandle,'GSM CUSD response: ' + s);
    end;
  until true; // ne ponavljaj nikad

  Result := True; //?
end;

procedure TfrmMain.mnuSetupClick(Sender: TObject);
var
  reg: TRegistry;
begin
  // popuni frmOpcije
  ini:=TIniFile.Create(appPath+'\opcije.ini');

  frmOpcije.txtRemoteHandle.Text := ini.ReadString('Skype','RemoteHandle','');
  frmOpcije.ckSilentMode.Checked := ini.ReadBool('Skype','SilentMode',false);
  frmOpcije.ckStartSkype.Checked := ini.ReadBool('Skype','StartSkype',false);

  // prikazi autostart
  reg := TRegistry.Create();
  reg.RootKey := HKEY_LOCAL_MACHINE;
  reg.OpenKeyReadOnly('SOFTWARE\Microsoft\Windows\CurrentVersion\Run');
  if reg.ReadString('SkypeDivert') <> '' then
    frmOpcije.ckStartWithWindows.Checked := true
  else
    frmOpcije.ckStartWithWindows.Checked := false;
  reg.CloseKey;
  reg.Free;

  frmOpcije.ShowModal;
  if frmOpcije.ModalResult = mrOk then
  begin
    // dohvati sa frmOpcije pa snimi u INI
    ini.WriteString('Skype','RemoteHandle',frmOpcije.txtRemoteHandle.Text);
    ini.WriteBool('Skype','SilentMode',frmOpcije.ckSilentMode.Checked);
    ini.WriteBool('Skype','StartSkype',frmOpcije.ckStartSkype.Checked);

    ini.UpdateFile;

    // reload runtime options...
    Opcije.SkypeOpt.RemoteHandle := frmOpcije.txtRemoteHandle.Text;

    // sredi autostart
    reg := TRegistry.Create();
    reg.RootKey := HKEY_LOCAL_MACHINE;
    reg.OpenKey('SOFTWARE\Microsoft\Windows\CurrentVersion\Run',false);
    if frmOpcije.ckStartWithWindows.Checked then
    begin
      // kreiraj ga
      reg.WriteString('SkypeDivert',ParamStr(0));
    end else
    begin
      // obrisi ga
      reg.DeleteValue('SkypeDivert');
    end;
    reg.CloseKey;
    reg.Free;
  end;

  ini.Free;
end;

procedure TfrmMain.mnuAboutClick(Sender: TObject);
begin
  ShowMessage('Skype Divert'+#13#10#13#10+'by www.elektronika.ba');
end;

end.
