unit uModemList;

interface

uses
  uModemItem,
  Winapi.Windows, Generics.Collections;

type
  TModemList = class
  private
    FItems: TObjectList<TModemItem>;
  public
    constructor Create;
    destructor Destroy; override;

    procedure Refresh;
    function Find(const AModemPort: String; var AModem: TModemItem): Boolean;

    property Items: TObjectList<TModemItem> read FItems;
  end;

implementation

uses
  System.Win.Registry, System.Classes, System.SysUtils;


{ TModemList }

constructor TModemList.Create;
begin
  FItems := TObjectList<TModemItem>.Create;
end;

destructor TModemList.Destroy;
begin
  FItems.Free;

  inherited;
end;

function TModemList.Find(const AModemPort: String; var AModem: TModemItem): Boolean;
var
  modem: TModemItem;
begin
  for modem in FItems do
    if modem.AttachedTo = AModemPort then
    begin
      AModem := modem;
      Exit(TRUE);
    end;

  Exit(FALSE);
end;

procedure TModemList.Refresh;
const
  MODEM_CLASS_KEY = 'SYSTEM\CurrentControlSet\Control\Class\{4D36E96D-E325-11CE-BFC1-08002BE10318}';
var
  reg : TRegistry;
  keys: TStringList;
  C1  : Integer;
begin
  FItems.Clear;

  reg := TRegistry.Create;
  try
    reg.RootKey := HKEY_LOCAL_MACHINE;

    keys := TStringList.Create;
    try
      if reg.OpenKeyReadOnly(MODEM_CLASS_KEY) then
      begin
        reg.GetKeyNames(keys);
        reg.CloseKey;
      end;

      for C1 := 0 to keys.Count - 1 do
        if reg.OpenKeyReadOnly(Format('%s\%s', [MODEM_CLASS_KEY, keys[C1]])) then
        begin
          FItems.Add(TModemItem.Create(reg.ReadString('MatchingDeviceId'),
                                       reg.ReadString('FriendlyName'),
                                       reg.ReadString('AttachedTo')));
          reg.CloseKey;
        end;

    finally
      keys.Free;
    end;
  finally
    reg.Free;
  end;
end;


end.
