unit uSerialCommReader;

interface

uses
  Winapi.Windows, System.Classes;

type
  TSerialCommReader = class(TThread)
  private
    FPortHandle     : THandle;
    FNotifyHandle   : THandle;
    FWMDataReceive  : UINT;
    FBufferSize     : DWORD;
    FCriticalSection: TRTLCriticalSection;
    FData           : String;

  protected
    procedure Execute; override;

  public
    constructor Create(const APortHandle: THandle; const ABufferSize: DWORD; const ANotifyHandle: THandle; const AWMDataReceive: UINT);
    destructor Destroy; override;

    function GetData: String;

    procedure Shutdown;

    property BufferSize: DWORD read FBufferSize;
  end;

implementation


constructor TSerialCommReader.Create(const APortHandle: THandle; const ABufferSize: DWORD; const ANotifyHandle: THandle; const AWMDataReceive: UINT);
begin
  inherited Create(TRUE);

  FBufferSize := ABufferSize;
  FPortHandle := APortHandle;
  FNotifyHandle := ANotifyHandle;
  FWMDataReceive := AWMDataReceive;

  InitializeCriticalSection(FCriticalSection);
end;

destructor TSerialCommReader.Destroy;
begin
  DeleteCriticalSection(FCriticalSection);

  inherited;
end;

procedure TSerialCommReader.Execute;
var
  buffer: PAnsiChar;
  bRead : DWORD;
  data  : String;
begin
  buffer := AllocMem(FBufferSize + 1);
  try
    while not Terminated do
    begin
      ReadFile(FPortHandle, buffer[0], FBufferSize, bRead, nil);

      if bRead > 0 then
      begin
        buffer[bRead] := #0;
        data := String(buffer);
        EnterCriticalSection(FCriticalSection);
        try
          FData := FData + data;
        finally
          LeaveCriticalSection(FCriticalSection);
        end;
        PostMessage(FNotifyHandle, FWMDataReceive, 0, 0);
      end;

      Sleep(10);
    end;
  finally
    FreeMem(buffer);
  end;
end;

function TSerialCommReader.GetData: String;
begin
  EnterCriticalSection(FCriticalSection);
  try
    result := FData;
    FData := '';
  finally
    LeaveCriticalSection(FCriticalSection);
  end;
end;

procedure TSerialCommReader.Shutdown;
begin
  Terminate;
  WaitFor;
end;

end.
