unit uSettings;

interface

uses
  superobject;

type
  TSettings = class
  private
    const
      JSON_MODEM_PORT       = 'modem_port';
      JSON_MODEM_SCRIPT     = 'modem_script';
      JSON_SKYPE_ID         = 'skype_id';
      JSON_SKYPE_AUTORUN    = 'skype_autorun';
      JSON_SKYPE_SILENTMODE = 'skype_silentmode';


    var
      FJSON        : ISuperObject;
      FSettingsFile: String;

    function GetModemPort: String;
    procedure SetModemPort(const AValue: String);

    function GetModemScript: String;
    procedure SetModemScript(const AValue: String);

    function GetSkypeId: String;
    procedure SetSkypeId(const AValue: String);

    function GetSkypeAutorun: Boolean;
    procedure SetSkypeAutorun(const AValue: Boolean);

    function GetSkypeSilentMode: Boolean;
    procedure SetSkypeSilentMode(const AValue: Boolean);

  public
    constructor Create(const ASettingsFile: String);

    procedure LoadSettings;
    procedure SaveSettings;

    property SettingsFile   : String read FSettingsFile;
    property ModemPort      : String read GetModemPort write SetModemPort;
    property ModemScript    : String read GetModemScript write SetModemScript;
    property SkypeId        : String read GetSkypeId write SetSkypeId;
    property SkypeAutorun   : Boolean read GetSkypeAutorun write SetSkypeAutorun;
    property SkypeSilentMode: Boolean read GetSkypeSilentMode write SetSkypeSilentMode;
  end;


implementation

uses
  System.SysUtils;


{ TSettings }

constructor TSettings.Create(const ASettingsFile: String);
begin
  FSettingsFile := ASettingsFile;
end;

procedure TSettings.LoadSettings;
begin
  if not FileExists(FSettingsFile) then
    SaveSettings;

  FJSON := TSuperObject.ParseFile(FSettingsFile, FALSE);
end;

procedure TSettings.SaveSettings;
begin
  if not Assigned(FJSON) then
    FJSON := so;

  FJSON.SaveTo(FSettingsFile);
end;


// GETTERS /////////////////////////////////////////////////////////////////////

function TSettings.GetModemPort: String;
begin
  result := FJSON.S[JSON_MODEM_PORT];
end;

function TSettings.GetModemScript: String;
begin
  result := FJSON.S[JSON_MODEM_SCRIPT];
end;

function TSettings.GetSkypeId: String;
begin
  result := FJSON.S[JSON_SKYPE_ID];
end;


function TSettings.GetSkypeAutorun: Boolean;
begin
  result := FJSON.B[JSON_SKYPE_AUTORUN];
end;

function TSettings.GetSkypeSilentMode: Boolean;
begin
  result := FJSON.B[JSON_SKYPE_SILENTMODE];
end;


// SETTERS /////////////////////////////////////////////////////////////////////

procedure TSettings.SetModemPort(const AValue: String);
begin
  FJSON.S[JSON_MODEM_PORT] := AValue;
end;

procedure TSettings.SetModemScript(const AValue: String);
begin
  FJSON.S[JSON_MODEM_SCRIPT] := AValue;
end;

procedure TSettings.SetSkypeId(const AValue: String);
begin
  FJSON.S[JSON_SKYPE_ID] := AValue;
end;

procedure TSettings.SetSkypeAutorun(const AValue: Boolean);
begin
  FJSON.B[JSON_SKYPE_AUTORUN] := AValue;
end;

procedure TSettings.SetSkypeSilentMode(const AValue: Boolean);
begin
  FJSON.B[JSON_SKYPE_SILENTMODE] := AValue;
end;


end.

