unit uMainForm;

interface

uses
  SKYPE4COMLib_TLB,
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, cxGraphics, cxControls, cxLookAndFeels,
  cxLookAndFeelPainters, cxContainer, cxEdit, dxSkinsCore,
  dxSkinDevExpressDarkStyle, Vcl.Menus, Vcl.ExtCtrls, CoolTrayIcon,
  Vcl.StdCtrls, cxButtons, dxGDIPlusClasses;

const
  WM_MYMEMO_ENTER = WM_USER + 500;

type
  TState = (stIdle, stOpenPort, stOpenPortIdle, stInitializeModem, stInitializeModemIdle,
            stSkypeStart, stSkypeStartIdle, stSkypeAttach, stSkypeAttachIdle, stConnected);

  TfrmMain = class(TForm)
    TrayIcon: TCoolTrayIcon;
    btSettings: TcxButton;
    imgLogo: TImage;
    meLog: TMemo;
    imgModem: TImage;
    imgSkype: TImage;
    btAbout: TcxButton;
    shHeader: TShape;
    pmTray: TPopupMenu;
    miSettings: TMenuItem;
    N1: TMenuItem;
    miExit: TMenuItem;
    tiIterator: TTimer;
    shActivity: TShape;
    procedure TrayIconStartup(Sender: TObject; var ShowMainForm: Boolean);
    procedure TrayIconMinimizeToTray(Sender: TObject);
    procedure TrayIconClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btSettingsClick(Sender: TObject);
    procedure meLogEnter(Sender: TObject);
    procedure meLogExit(Sender: TObject);
    procedure meLogChange(Sender: TObject);
    procedure btAboutClick(Sender: TObject);
    procedure miSettingsClick(Sender: TObject);
    procedure miExitClick(Sender: TObject);
    procedure tiIteratorTimer(Sender: TObject);
  private
    FState: TState;

    procedure OnModemOpenPort(Sender: TObject; const AReturnCode: Integer);
    procedure OnModemInitialize(Sender: TObject);

    procedure OnSkypeAttachmentStatus(ASender: TObject; Status: TAttachmentStatus);

    procedure Log(const AText: String);
    procedure LogLn(const AText: String);

    procedure OpenPort;
    procedure InitializeModem;
    procedure SkypeStart;
    procedure SkypeAttach;

  protected
    procedure WMMYMEMOENTER(var Message: TMessage); message WM_MYMEMO_ENTER;

  public

  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

uses
  uMainDataModule, uModemItem, uCommon,
  System.Win.Registry, uSettingsForm, uAboutForm;


procedure TfrmMain.FormCreate(Sender: TObject);
begin
  FState := stOpenPort;

  LoadImageFromResource(imgModem, RESOURCE_MODEM_OFF);
  LoadImageFromResource(imgSkype, RESOURCE_SKYPE_OFF);

  dmMain.ModemCommunication.OnOpenPort := OnModemOpenPort;
  dmMain.ModemCommunication.OnInitialize := OnModemInitialize;

  dmMain.Skype.OnAttachmentStatus := OnSkypeAttachmentStatus;
end;

procedure TfrmMain.OpenPort;
var
  modem: TModemItem;
begin
  if dmMain.Modems.Find(dmMain.Settings.ModemPort, modem) then
  begin
    if dmMain.ModemCommunication.Serial.Active then
    begin
      Log(Format('Closing port [%s]...', [dmMain.ModemCommunication.Serial.Port]));
      dmMain.ModemCommunication.Serial.Stop;
      LogLn(' OK');
    end;

    dmMain.ModemCommunication.ClearQueue;
    Log(Format('Opening port [%s]...', [modem.AttachedTo]));
    dmMain.ModemCommunication.OpenPort(modem.AttachedTo);
    FState := stOpenPortIdle;
  end
  else
  begin
    LogLn('Cannot find the modem. Please check the settings.');
    FState := stIdle;
  end;
end;

procedure TfrmMain.InitializeModem;
begin
  Log(Format('Initializing modem [%s]...', [dmMain.ModemCommunication.Serial.Port]));
  dmMain.ModemCommunication.Initialize;
  FState := stInitializeModemIdle;
end;

procedure TfrmMain.SkypeStart;
begin
  if not dmMain.Skype.Client.IsRunning then
  begin
    if FState <> stSkypeStartIdle then
    begin
      if dmMain.Settings.SkypeAutorun then
      begin
        Log('Starting Skype...');
        dmMain.Skype.Client.Start(TRUE, TRUE);
        FState := stSkypeStartIdle
      end
      else
        LogLn('ERROR: Skype is not running');
    end;
  end
  else
  begin
    if FState = stSkypeStartIdle then
      LogLn(' OK');

    FState := stSkypeAttach;
    tiIterator.OnTimer(nil);
  end;
end;

procedure TfrmMain.SkypeAttach;
begin
  if dmMain.Skype.Client.IsRunning then
  begin
    Log('Attaching to Skype...');
    if dmMain.Skype.AttachmentStatus <> apiAttachSuccess then
    begin
      dmMain.Skype.Attach(dmMain.Skype.Protocol, FALSE);
      FState := stSkypeAttachIdle;
    end
    else
    begin
      LogLn(' OK [already attached]');
      FState := stConnected;
      tiIterator.OnTimer(nil);
    end;
  end
  else
    FState := stSkypeStart;
end;

procedure TfrmMain.tiIteratorTimer(Sender: TObject);
begin
  if FState <> stConnected then
    shActivity.Brush.Color := clRed
  else
    shActivity.Brush.Color := clLime;

  case FState of
    stOpenPort       : begin
                         OpenPort;
                         tiIterator.Interval := 2000;
                       end;

    stInitializeModem: begin
                         InitializeModem;
                         tiIterator.Interval := 1000;
                       end;

    stSkypeStart,
    stSkypeStartIdle : begin
                         SkypeStart;
                         tiIterator.Interval := 3000;
                       end;

    stSkypeAttach    : begin
                         SkypeAttach;
                         tiIterator.Interval := 1000;
                       end;

    stConnected      : begin
                         shActivity.Brush.Color := clLime;
                         tiIterator.Interval := 500;
                       end;
  end;
end;

procedure TfrmMain.Log(const AText: String);
begin
  meLog.Text := meLog.Text + AText;
  SendMessage(meLog.Handle, EM_LINESCROLL, 0, meLog.Lines.Count);
  meLog.Refresh;
end;

procedure TfrmMain.LogLn(const AText: String);
begin
  Log(AText + #13#10);
end;

procedure TfrmMain.btAboutClick(Sender: TObject);
begin
  TfrmAbout.RunModal;
  PostMessage(Handle, WM_MYMEMO_ENTER, 0, 0);
end;

procedure TfrmMain.btSettingsClick(Sender: TObject);
var
  old_modem_port, old_modem_script: String;
begin
  old_modem_port := dmMain.Settings.ModemPort;
  old_modem_script := dmMain.Settings.ModemScript;

  if TfrmSettings.RunModal = mrOk then
  begin
    dmMain.Settings.SaveSettings;

    if (old_modem_port <> dmMain.Settings.ModemPort) or
       (old_modem_script <> dmMain.Settings.ModemScript) then
    begin
      if old_modem_script <> dmMain.Settings.ModemScript then
        dmMain.ModemCommunication.SetScript(dmMain.Settings.ModemScript);
      FState := stOpenPort;
    end;
  end;

  PostMessage(Handle, WM_MYMEMO_ENTER, 0, 0);
end;

procedure TfrmMain.TrayIconClick(Sender: TObject);
begin
  TrayIcon.ShowMainForm;
end;

procedure TfrmMain.TrayIconMinimizeToTray(Sender: TObject);
begin
  TrayIcon.HideMainForm;
end;

procedure TfrmMain.TrayIconStartup(Sender: TObject; var ShowMainForm: Boolean);
begin
  TrayIcon.Icon.Assign(Application.Icon);
  TrayIcon.IconVisible := TRUE;
end;

procedure TfrmMain.WMMYMEMOENTER(var Message: TMessage);
begin
  CreateCaret(meLog.Handle, 0, 0, 0);
end;

procedure TfrmMain.meLogChange(Sender: TObject);
begin
  CreateCaret(meLog.Handle, 0, 0, 0);
end;

procedure TfrmMain.meLogEnter(Sender: TObject);
begin
  PostMessage(Handle, WM_MYMEMO_ENTER, 0, 0);
end;

procedure TfrmMain.meLogExit(Sender: TObject);
begin
  CreateCaret(meLog.Handle, 1, 1, 1);
end;

procedure TfrmMain.miExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.miSettingsClick(Sender: TObject);
begin
  btSettings.OnClick(Sender);
end;

procedure TfrmMain.OnModemOpenPort(Sender: TObject; const AReturnCode: Integer);
begin
  if AReturnCode = ERROR_SUCCESS then
  begin
    LogLn(' OK');
    FState := stInitializeModem;
    tiIterator.OnTimer(Sender);
  end
  else
  begin
    LogLn(Format(' ERROR [%d]', [AReturnCode]));
    FState := stOpenPort;
  end;
end;


procedure TfrmMain.OnModemInitialize(Sender: TObject);
begin
  LoadImageFromResource(imgModem, RESOURCE_MODEM_ON);
  LogLn(' OK');

  FState := stSkypeStart;
  tiIterator.OnTimer(Sender);
end;

procedure TfrmMain.OnSkypeAttachmentStatus(ASender: TObject; Status: TAttachmentStatus);
begin
  if FState = stSkypeAttachIdle then
  begin
    case Status of
      apiAttachSuccess: begin
                          if dmMain.Skype.ConnectionStatus = conOnline then
                          begin
                            LogLn(' OK');
                            dmMain.Skype.SilentMode := dmMain.Settings.SkypeSilentMode;
                            FState := stConnected;
                            tiIterator.OnTimer(nil)
                          end
                          else
                          begin
                            LogLn(Format(' ERROR [ConnectionStatus=%d]', [dmMain.Skype.ConnectionStatus]));
                            FState := stSkypeAttach;
                          end;
                        end;

      apiAttachPendingAuthorization: ;
      apiAttachAvailable: ;
    else
      LogLn(Format(' ERROR [Status=%d]', [Status]));
      FState := stSkypeAttach;
    end;
  end;

  if (dmMain.Skype.AttachmentStatus = apiAttachSuccess) and
     (dmMain.Skype.ConnectionStatus = conOnline) then
  begin
    LoadImageFromResource(imgSkype, RESOURCE_SKYPE_ON);
  end
  else
  begin
    LoadImageFromResource(imgSkype, RESOURCE_SKYPE_OFF);
    if (dmMain.Skype.AttachmentStatus <> apiAttachPendingAuthorization) and
       (dmMain.Skype.AttachmentStatus <> apiAttachAvailable) then
      FState := stSkypeAttach;
  end;
end;

end.
