unit uSettingsForm;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ComCtrls, Vcl.StdCtrls, Vcl.ExtCtrls,
  cxGraphics, cxControls, cxLookAndFeels, cxLookAndFeelPainters, cxContainer,
  cxEdit, dxSkinsCore, dxSkinDevExpressDarkStyle, cxLabel, Vcl.Menus;

type
  TfrmSettings = class(TForm)
    pcSettings: TPageControl;
    tsModem: TTabSheet;
    tsSkype: TTabSheet;
    paTop: TPanel;
    paLeft: TPanel;
    paRight: TPanel;
    paButtons: TPanel;
    btCancel: TButton;
    btOk: TButton;
    lbcModem: TcxLabel;
    cbModems: TComboBox;
    lbcScript: TcxLabel;
    btBrowseScript: TButton;
    lbcReceiver: TcxLabel;
    cbReceiver: TComboBox;
    OpenDialog: TOpenDialog;
    edModemScript: TEdit;
    btEditScript: TButton;
    cbAutorunSkype: TCheckBox;
    cbSkypeSilentMode: TCheckBox;
    tsGeneral: TTabSheet;
    cbStartWithWindows: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure cbModemsChange(Sender: TObject);
    procedure cbReceiverChange(Sender: TObject);
    procedure btBrowseScriptClick(Sender: TObject);
    procedure btEditScriptClick(Sender: TObject);
  private
    FSelectedModemPort: String;
    FSelectedSkypeId  : String;

    procedure RefreshModemList;
    procedure RefreshReceiversList;

    procedure SelectComboboxReceiver;
    function ExtractReceiverId(const AString: String): String;

    procedure SaveSettings;

  public
    class function RunModal: Integer;
  end;

var
  frmSettings: TfrmSettings;

implementation

{$R *.dfm}

uses
  uMainDataModule, uModemItem, SKYPE4COMLib_TLB, uStartup,
  ShellApi;


class function TfrmSettings.RunModal: Integer;
begin
  Vcl.Forms.Application.CreateForm(TfrmSettings, frmSettings);
  try
    result := frmSettings.ShowModal;
    if result = mrOk then
      frmSettings.SaveSettings;
  finally
    frmSettings.Free;
  end;
end;

////////////////////////////////////////////////////////////////////////////////

procedure TfrmSettings.FormCreate(Sender: TObject);
begin
  FSelectedModemPort := dmMain.Settings.ModemPort;
  FSelectedSkypeId := dmMain.Settings.SkypeId;

  cbStartWithWindows.Checked := TStartup.IsInstalled;
  cbAutorunSkype.Checked := dmMain.Settings.SkypeAutorun;
  cbSkypeSilentMode.Checked := dmMain.Settings.SkypeSilentMode;
  edModemScript.Text := dmMain.Settings.ModemScript;

  RefreshModemList;
  RefreshReceiversList;
end;

procedure TfrmSettings.RefreshModemList;
var
  C1        : Integer;
  item_index: Integer;
begin
  dmMain.Modems.Refresh;
  cbModems.Clear;
  item_index := -1;
  for C1 := 0 to dmMain.Modems.Items.Count - 1 do
  begin
    cbModems.AddItem(Format('%s: %s', [dmMain.Modems.Items[C1].AttachedTo, dmMain.Modems.Items[C1].FriendlyName]), dmMain.Modems.Items[C1]);
    if (FSelectedModemPort <> '') and
       (dmMain.Modems.Items[C1].AttachedTo = FSelectedModemPort) then
      item_index := C1;
  end;

  if item_index <> -1 then
    cbModems.ItemIndex := item_index;
end;

procedure TfrmSettings.RefreshReceiversList;
var
  C1  : Integer;
  user: IUser;
  text: String;
begin
  cbReceiver.Clear;
  if dmMain.Skype.Client.IsRunning then
    for C1 := 1 to dmMain.Skype.Friends.Count do
    begin
      user := dmMain.Skype.Friends.Item[C1];
      if user.DisplayName <> '' then
        text := Format('%s (%s)', [user.DisplayName, user.Handle])
      else
        if user.FullName <> '' then
          text := Format('%s (%s)', [user.FullName, user.Handle])
        else
          text := user.Handle;

      cbReceiver.AddItem(text, nil);
    end;

  SelectComboboxReceiver;
end;

procedure TfrmSettings.btBrowseScriptClick(Sender: TObject);
begin
  if OpenDialog.Execute(Handle) then
    edModemScript.Text := OpenDialog.FileName;
end;

procedure TfrmSettings.btEditScriptClick(Sender: TObject);
begin
  ShellExecute(0, nil, PChar(edModemScript.Text), nil, nil, SW_SHOWNORMAL);
end;

procedure TfrmSettings.cbModemsChange(Sender: TObject);
begin
  FSelectedModemPort := (cbModems.Items.Objects[cbModems.ItemIndex] as TModemItem).AttachedTo;
end;

procedure TfrmSettings.cbReceiverChange(Sender: TObject);
begin
  FSelectedSkypeId := ExtractReceiverId(cbReceiver.Text);
end;

function TfrmSettings.ExtractReceiverId(const AString: String): String;
var
  text: String;
  C1  : Integer;
begin
  text := AString;
  if (text <> '') and
     (text[Length(text)] = ')') then
  begin
    Delete(text, Length(text), 1);
    C1 := Length(text);
    while (C1 > 0) and
          (text[C1] <> '(') do
      Dec(C1);
    Delete(text, 1, C1);
  end;
  result := text;
end;

procedure TfrmSettings.SelectComboboxReceiver;
var
  C1: Integer;
begin
  for C1 := 0 to cbReceiver.Items.Count - 1 do
    if FSelectedSkypeId = ExtractReceiverId(cbReceiver.Items[C1]) then
    begin
      cbReceiver.ItemIndex := C1;
      Exit;
    end;
end;

procedure TfrmSettings.SaveSettings;
begin
  dmMain.Settings.ModemPort := FSelectedModemPort;
  dmMain.Settings.SkypeId := FSelectedSkypeId;
  dmMain.Settings.ModemScript := edModemScript.Text;
  dmMain.Settings.SkypeAutorun := cbAutorunSkype.Checked;
  dmMain.Settings.SkypeSilentMode := cbSkypeSilentMode.Checked;

  if cbStartWithWindows.Checked then
    TStartup.Install
  else
    if TStartup.IsInstalled then
      TStartup.Uninstall;
end;

end.
